<?php
/**
 * Floating Single Ad
 *
 * @package   Floating Single Ad
 * @author    Pete Williams <support@preneurgroup.com>
 * @license   GPL-2.0+
 * @link      http://preneurmarketing.com/
 * @copyright 2013 Preneur Marketing
 */

/**
 * Plugin class. This class should ideally be used to work with the
 * administrative side of the WordPress site.
 *
 * If you're interested in introducing public-facing
 * functionality, then refer to `class-floating-single-ad-admin.php`
 */
class Floating_Single_Ad_Admin {

	/**
	 * Instance of this class.
	 *
	 * @since    1.0.0
	 *
	 * @var      object
	 */
	protected static $instance = null;

	/**
	 * Slug of the plugin screen.
	 *
	 * @since    1.0.0
	 *
	 * @var      string
	 */
	protected $plugin_screen_hook_suffix = null;

	/**
	 * Initialize the plugin by loading admin scripts & styles and adding a
	 * settings page and menu.
	 *
	 * @since     1.0.0
	 */
	private function __construct() {

		/*
		 * Call $plugin_slug from public plugin class.
		 *
		 */
		$plugin = Floating_Single_Ad::get_instance();
		$this->plugin_slug = $plugin->get_plugin_slug();
		
		add_action( 'init', array( $this, 'create_post_type' ) );
		//add_action( 'init', array( $this, 'create_taxonomy' ) );

		// Load admin style sheet and JavaScript.
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_styles' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_scripts' ) );

		// Add the options page and menu item.
		add_action( 'admin_menu', array( $this, 'add_plugin_admin_menu' ) );

		/*
		 * Add an action link pointing to the options page.
		 *
		 * TODO:
		 *
		 * - Rename "floating-single-ad.php" to the name your plugin
		 */
		$plugin_basename = plugin_basename( plugin_dir_path( __FILE__ ) . 'floating-single-ad.php' );
		add_filter( 'plugin_action_links_' . $plugin_basename, array( $this, 'add_action_links' ) );

		/*
		 * Define custom functionality.
		 *
		 * Read more about actions and filters:
		 * http://codex.wordpress.org/Plugin_API#Hooks.2C_Actions_and_Filters
		 */
		// add_action( 'TODO', array( $this, 'action_floating_single_ad' ) );
		add_filter( 'manage_edit-fsa_columns', array( $this, 'set_custom_edit_fsa_columns' ) );
		add_action( 'manage_fsa_posts_custom_column', array( $this, 'posts_custom_column' ), 10, 2 );
		add_filter( 'manage_edit-fsa_sortable_columns', array( $this, 'fsa_sortable_columns' ) );
		add_action( 'load-edit.php', array( $this, 'edit_fsa_load' ) );

	}

	/**
	 * Return an instance of this class.
	 *
	 * @since     1.0.0
	 *
	 * @return    object    A single instance of this class.
	 */
	public static function get_instance() {

		// If the single instance hasn't been set, set it now.
		if ( null == self::$instance ) {
			self::$instance = new self;
		}

		return self::$instance;
	}
	
	// Create the post type.
	public function create_post_type() {
		$labels = array(
		    'name' => __( 'Floating Ads', $this->plugin_slug ),
		    'singular_name' => __( 'Floating Ad', $this->plugin_slug ),
		    'add_new' => __( 'Add New Floating Ad', $this->plugin_slug ),
		    'all_items' => __( 'All Floating Ads', $this->plugin_slug ),
		    'add_new_item' => __( 'Add New Floating Ad', $this->plugin_slug ),
		    'edit_item' => __( 'Edit Floating Ad', $this->plugin_slug ),
		    'new_item' => __( 'New Floating Ad', $this->plugin_slug ),
		    'view_item' => __( 'View Floating Ad', $this->plugin_slug ),
		    'search_items' => __( 'Search Floating Ads', $this->plugin_slug ),
		    'not_found' =>  __( 'No floating ads found', $this->plugin_slug ),
		    'not_found_in_trash' => __( 'No floating ads found in trash', $this->plugin_slug ),
		    'parent_item_colon' => __( 'Parent Floating Ad:', $this->plugin_slug ),
		    'menu_name' => __( 'Floating Ads', $this->plugin_slug )
		);
		$args = array(
			'labels' => $labels,
			'description' => "A content type for adding advertisements.",
			'public' => true,
			'exclude_from_search' => true,
			'publicly_queryable' => false,
			'show_ui' => true, 
			'show_in_nav_menus' => true, 
			'show_in_menu' => true,
			'show_in_admin_bar' => true,
			'menu_position' => 15,
			'menu_icon' => plugins_url( 'images/menu-icon.png', __FILE__ ),
			'capability_type' => 'post',
			'hierarchical' => false,
			'supports' => array( 'title', 'editor', 'thumbnail', 'custom-fields', 'revisions', 'page-attributes' ),
			'has_archive' => false,
			'rewrite' => false,
			'query_var' => false,
			'can_export' => true
		); 
		register_post_type( 'fsa', $args);
	}

	/**
	 * Register and enqueue admin-specific style sheet.
	 *
	 * @since     1.0.0
	 *
	 * @return    null    Return early if no settings page is registered.
	 */
	public function enqueue_admin_styles() {

		if ( ! isset( $this->plugin_screen_hook_suffix ) ) {
			return;
		}

		$screen = get_current_screen();
		if ( $this->plugin_screen_hook_suffix == $screen->id ) {
			wp_enqueue_style( $this->plugin_slug .'-admin-styles', plugins_url( 'css/admin.css', __FILE__ ), array(), Floating_Single_Ad::VERSION );
		}

	}

	/**
	 * Register and enqueue admin-specific JavaScript.
	 *
	 * @since     1.0.0
	 *
	 * @return    null    Return early if no settings page is registered.
	 */
	public function enqueue_admin_scripts() {

		if ( ! isset( $this->plugin_screen_hook_suffix ) ) {
			return;
		}

		$screen = get_current_screen();
		if ( $this->plugin_screen_hook_suffix == $screen->id ) {
			wp_enqueue_script( $this->plugin_slug . '-admin-script', plugins_url( 'js/admin.js', __FILE__ ), array( 'jquery' ), Floating_Single_Ad::VERSION );
		}

	}

	/**
	 * Register the administration menu for this plugin into the WordPress Dashboard menu.
	 *
	 * @since    1.0.0
	 */
	public function add_plugin_admin_menu() {

		/*
		 * Add a settings page for this plugin to the Settings menu.
		 *
		 * NOTE:  Alternative menu locations are available via WordPress administration menu functions.
		 *
		 *   Administration Menus: http://codex.wordpress.org/Administration_Menus
		 *   For reference: http://codex.wordpress.org/Roles_and_Capabilities
		 */
		$this->plugin_screen_hook_suffix = add_submenu_page(
			'edit.php?post_type=fsa',
			__( 'Single Floating Ad Settings', $this->plugin_slug ),
			__( 'Settings', $this->plugin_slug ),
			'edit_posts',
			$this->plugin_slug,
			array( $this, 'display_plugin_admin_page' )
		);
		
		// Optionally load check for saving plugin options.
        if ( isset( $_REQUEST['fsa-plugin-save'] ) && $_REQUEST['fsa-plugin-save'] )
            add_action( 'admin_init', array( $this, 'save' ) );
	}
	
	// This adds custom fields to the list of ads here: edit.php?post_type=fsa
   	public function set_custom_edit_fsa_columns($columns) {
		return $columns + array(
			'start_date' => __('Start Date'), 
			'end_date' => __('End Date'),
			'impression_count' => __('Impression Count'),
			'impression_limit' => __('Impression Limit'),
			'click_count' => __('Click Count'),
			'ad_url' => __('URL'),
			'ad' => __('Ad'), 
		);
	}
	
	// This determines what to show in each column on the edit.php?post_type=fsa page.
	public function posts_custom_column( $column, $post_id ) {
		switch ( $column ) {
			case 'ad':
				echo get_post($post_id)->post_content; 
				break;

			case 'start_date':
				$start_date = get_post_meta( $post_id , '_fsa_start_date' , true ); 
				if ( $start_date != '') {
					echo date_i18n( 'm/d/Y', $start_date );
				} else {
					echo 'Immediately';
				}
				break;

			case 'end_date':
				$end_date = get_post_meta( $post_id , '_fsa_end_date' , true ); 
				if ( $end_date != '') {
					echo date_i18n( 'm/d/Y', $end_date );
				} else {
					echo 'Never';
				}
				break;

			case 'impression_count':
				$count = get_post_meta( $post_id , '_fsa_impression_count' , true ); 
				$count = intval( $count );
				echo number_format( $count );
				break;

			case 'impression_limit':
				$limit = get_post_meta( $post_id , '_fsa_impression_limit' , true );
				if ( $limit > 0 ) {
					echo number_format( $limit );
				} else {
					echo 'Unlimited';
				}
				break;
				
			case 'ad_url':
				$ad_url = get_post_meta( $post_id , '_fsa_ad_url' , true );
				echo $ad_url;
				break;
				
			case 'click_count':
				$click_count = get_post_meta( $post_id , '_fsa_click_count' , true );
				if ( $click_count > 0 ) {
					echo number_format( $click_count );
				} else {
					echo '0';
				}
				break;
		} // switch ( $column ) {
	}
	
		// This initializes sorting on the custom field columns.
	public function fsa_sortable_columns( $columns ) {
		$columns['start_date'] = '_fsa_start_date';
		$columns['end_date'] = '_fsa_end_date';
		$columns['impression_count'] = '_fsa_impression_count';
		$columns['impression_limit'] = '_fsa_impression_limit';
		$columns['click_count'] = '_fsa_click_count';
		return $columns;
	}
	
	// This captures the $_REQUEST so we can modify the query for sortable columns.
	function edit_fsa_load() {
		add_filter( 'request', array( $this, 'fsa_sort_custom_fields' ) );
	}

	// Figure out what columns we are sorting on so we can update the query.
	function fsa_sort_custom_fields( $vars ) {

		if ( isset( $vars['post_type'] ) && 'fsa' == $vars['post_type'] ) {
			
			// start date
			if ( isset( $vars['orderby'] ) && '_fsa_start_date' == $vars['orderby'] ) {
				$vars = array_merge(
					$vars,
					array(
						'meta_key' => '_fsa_start_date',
						'orderby' => 'meta_value_num'
					)
				);
			}
			
			// end date
			if ( isset( $vars['orderby'] ) && '_fsa_end_date' == $vars['orderby'] ) {
				$vars = array_merge(
					$vars,
					array(
						'meta_key' => '_fsa_end_date',
						'orderby' => 'meta_value_num'
					)
				);
			}
			
			// impression count
			if ( isset( $vars['orderby'] ) && '_fsa_impression_count' == $vars['orderby'] ) {
				$vars = array_merge(
					$vars,
					array(
						'meta_key' => '_fsa_impression_count',
						'orderby' => 'meta_value_num'
					)
				);
			}
			
			// impression limit
			if ( isset( $vars['orderby'] ) && '_fsa_impression_limit' == $vars['orderby'] ) {
				$vars = array_merge(
					$vars,
					array(
						'meta_key' => '_fsa_impression_limit',
						'orderby' => 'meta_value_num'
					)
				);
			}	
			
			// Click count
			if ( isset( $vars['orderby'] ) && '_fsa_click_count' == $vars['orderby'] ) {
				$vars = array_merge(
					$vars,
					array(
						'meta_key' => '_fsa_click_count',
						'orderby' => 'meta_value_num'
					)
				);
			}		
		}

		return $vars;
	}
	
	
	 /**
     * Saves the plugin options if the user has submitted them.
     *
     * @since 1.0.1
     */
    public function save() {

        // Do a security check before saving options first.
        check_admin_referer( 'fsa_save_options', 'fsa_save_options' );

        // If no data was submitted, return early.
        if ( empty( $_REQUEST['_fsa_data'] ) )
            return;

        // Grab the plugin options and data that has been submitted.
        $option    = get_option( 'fsa_global_option' );
        $submitted = stripslashes_deep( $_REQUEST['_fsa_data'] );

        // Unset the submit value since we don't need it.
        if ( isset( $submitted['submit'] ) )
            unset( $submitted['submit'] );

// Reset the show_on key.
        $option['show_on'] = array();
        $i = 0;

        foreach ( get_post_types( array( 'public' => true ) ) as $post_type )
            if ( isset( $submitted['show_on'] ) && in_array( $post_type, $submitted['show_on'] ) )
                $option['show_on'][] = esc_attr( $post_type );

        // Unset the show_on key from the submitted info.
        if ( isset( $submitted['show_on'] ) )
            unset( $submitted['show_on'] );

        // Sanitize the option values.
		$submitted['fsa_show_homepage'] 	= isset( $submitted['fsa_show_homepage'] ) ? 1 : 0;
		$submitted['fsa_show_post'] 		= isset( $submitted['fsa_show_post'] ) ? 1 : 0;
		$submitted['fsa_show_page'] 		= isset( $submitted['fsa_show_page'] ) ? 1 : 0;
		$submitted['fsa_width'] 			= isset( $submitted['fsa_width'] ) ? sanitize_text_field( $submitted['fsa_width'] ) : 0;
		$submitted['fsa_height'] 			= isset( $submitted['fsa_height'] ) ? sanitize_text_field( $submitted['fsa_height'] ) : 0;
		$submitted['fsa_position'] 			= isset( $submitted['fsa_position'] ) ? sanitize_text_field( $submitted['fsa_position'] ) : '';
		$submitted['fsa_top_margin'] 		= isset( $submitted['fsa_top_margin'] ) ? sanitize_text_field( $submitted['fsa_top_margin'] ) : 0;
		$submitted['fsa_left_margin'] 		= isset( $submitted['fsa_left_margin'] ) ? sanitize_text_field( $submitted['fsa_left_margin'] ) : 0;
		$submitted['fsa_right_margin'] 		= isset( $submitted['fsa_right_margin'] ) ? sanitize_text_field( $submitted['fsa_right_margin'] ) : 0;
		$submitted['fsa_bottom_margin'] 	= isset( $submitted['fsa_bottom_margin'] ) ? sanitize_text_field( $submitted['fsa_bottom_margin'] ) : 0;

        // Finally, update the option.
        update_option( 'fsa_global_option', array_merge( $option, $submitted ) );

    }
	

	/**
	 * Render the settings page for this plugin.
	 *
	 * @since    1.0.0
	 */
	public function display_plugin_admin_page() {
		include_once( 'views/admin.php' );
	}

	/**
	 * Add settings action link to the plugins page.
	 *
	 * @since    1.0.0
	 */
	public function add_action_links( $links ) {

		return array_merge(
			array(
				'settings' => '<a href="' . admin_url( 'edit.php?post_type=fsa&page=' . $this->plugin_slug ) . '">' . __( 'Settings', $this->plugin_slug ) . '</a>'
			),
			$links
		);

	}

	/**
	 * NOTE:     Actions are points in the execution of a page or process
	 *           lifecycle that WordPress fires.
	 *
	 *           Actions:    http://codex.wordpress.org/Plugin_API#Actions
	 *           Reference:  http://codex.wordpress.org/Plugin_API/Action_Reference
	 *
	 * @since    1.0.0
	 */
	public function action_floating_single_ad() {
		// TODO: Define your action hook callback here
	}

	/**
	 * NOTE:     Filters are points of execution in which WordPress modifies data
	 *           before saving it or sending it to the browser.
	 *
	 *           Filters: http://codex.wordpress.org/Plugin_API#Filters
	 *           Reference:  http://codex.wordpress.org/Plugin_API/Filter_Reference
	 *
	 * @since    1.0.0
	 */
	public function filter_floating_single_ad() {
		// TODO: Define your filter hook callback here
	}

}